#!/usr/bin/perl -w
# $Id: TechnoratiTags.pl,v 1.2 2005/05/06 02:58:38 george Exp $

package MT::Plugins::TechnoratiTags;
use strict;

use vars qw( $VERSION );
$VERSION = 0.3;

=head1 NAME

TechnoratiTags - Plugin for Moveable Type

=head1 DESCRIPTION

A variable plugin that outputs an entry's keywords as Technorati Tags.  For 
more information see http://www.technorati.com/help/tags.html

Technorati already adds tags for entry categories but this allows you to add
free-form tags for each entry.

=head1 REQUIREMENTS

Moveable Type 2.2 or higher

Tested on Movable Type 2.661 and 3.14

=head1 INSTALLATION

Put TechnoratiTags.pl in the plugins directory for your Moveable Type 
installation and make sure that the permissions make it readable by the web 
server.

Now, make sure that Movable Type is pinging Technorati by following the 
instructions at 
http://www.technorati.com/developers/pingconfig.html#head-47a9faa5390ea56a6c23fe7f74b4fc693f915465

You may also want to customize your New Entry form to display the keyword 
field.  Choose B<New Entry> then at the bottom of the form click the link 
B<Customize the display of this page.>  On the form that pops up choose
B<Custom> and then check all the fields you want to display, including 
B<Keywords>.  Click the B<Save> button.

=head1 USAGE

Add keywords for your post separated by spaces.  For example, the 
TechnoratiTags plugin might get these tags: B<technorati tags MovableType>

Note that B<Movable Type> was turned into one word: B<MovableType>.  If it
had been spelled with the space, each word would have been tagged with 
B<Movable> and B<Type> but not B<Movable Type>.

In your template add 

C<< <$MTTechnoratiTags$> >>

where you want to display tags.  By default it outputs the links with the 
class "TechnoratiTags" but you can optionally override this with the "class" 
attribute:

C<< <$MTTechnoratiTags class="someotherclass"$> >>

For more advanced formatting use Brad Choate's IfEmpty plugin from
http://www.bradchoate.com/weblog/2004/10/20/mtifempty

For example:

C<< <MTIfNotEmpty var="EntryKeywords"> >>

C<< <div id="some_id"> >>

C<< <h2>Tags</h2> >>

C<< <p><$MTTechnoratiTags$></p> >>

C<< </div> >>

C<< <MTIfNotEmpty var="MTEntryKeywords">  >>

=head1 CHANGES

=over

=item 0.2

There was a bug when an entry didn't have keywords that is now fixed.

=back

=head1 BUGS

I can't tell from Technorati's site whether the tags need to be on the 
individual entry archive template or the main index template or what.

=head1 COPYRIGHT

This program is copyright 2005 George Hotelling http://george.hotelling.net/
and licensed under the GPL General Public License.

=cut
use MT;
use SelfLoader;
use MT::Template::Context;

MT::Template::Context->add_tag('TechnoratiTags' => \&technoratiTags);
MT::Template::Context->add_container_tag('EntryKeywordsLoop', \&entryKeywordsLoop);
MT::Template::Context->add_tag('LoopedKeyword', \&loopedKeyword);
MT::Template::Context->add_tag('FormattedKeywords', \&formatKeywords);

1;

__DATA__


sub technoratiTags {
  my ($ctx, $args) = @_;
  my $class = ($args->{class})?$args->{class}:"TechnoratiTags";
  my $output = "";
  foreach my $tag (&_getKeywords($ctx)) {
  	$output .= "<a href=\"http://technorati.com/tag/$tag\" rel=\"tag\" class=\"$class\">$tag</a> ";
  }
  return $output
}

sub entryKeywordsLoop {
	my ($ctx, $args) = @_;
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    my $output = "";
    foreach my $keyword (&_getKeywords($ctx)) {
    	$ctx->stash('tt_looped_keyword', $keyword);
    	defined(my $built = $builder->build($ctx, $tokens))
    	  or return $ctx->error($builder->errstr);
        $output .= $built;    	
	}
	return $output;
}

sub loopedKeyword {
	my ($ctx, $args) = @_;
	return $ctx->stash('tt_looped_keyword');
}

sub formatKeywords {
	my ($ctx, $args) = @_;
	my @keywords = &_getKeywords($ctx);
	return "" unless @keywords;
	if $args->{'format'} {
		for (my $i=0; $i <= $#keywords; $i++) {
			$keywords[$i] = sprintf($args->{'format'}, $keywords);
		}
	}
	return $keywords[0] if (@keywords == 1);
	my $glue = $args->{'glue'} || "";
	return join $glue, @keywords;
}
		
	
	
sub _getKeywords {
	my ($ctx) = @_;
	my $entry = $ctx->stash('entry');
	return () unless my $keywords = $entry->keywords;
	return split /[, ]+/, $entry->keywords;
}


__END__
